# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import BaseModel, HashObject, ManifestModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import (
        DestinyActivityDefinition,
        DestinyActivityDifficultyId,
        DestinyActivityDifficultyTierType,
        DestinyActivityModifierConnotation,
        DestinyActivityModifierDisplayCategory,
        DestinyActivitySkullDynamicUse,
        DestinyDisplayPropertiesDefinition,
        DestinyTraitDefinition,
        DestinyUnlockExpressionDefinition,
    )


@custom_define()
class DestinyActivityFamilyDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    None
    Attributes:
        disabled_skull_category_hashes: _No description given by bungie._
        disabled_skull_subcategory_hashes: _No description given by bungie._
        fixed_skull_subcategory_hashes: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        traits: _No description given by bungie._
    """

    disabled_skull_category_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    disabled_skull_subcategory_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    fixed_skull_subcategory_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    index: int = custom_field()
    redacted: bool = custom_field()
    traits: list[int] = custom_field(metadata={"type": """list[int]"""})


@custom_define()
class DestinyActivitySkullCategoryDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    None
    Attributes:
        display_properties: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
    """

    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    index: int = custom_field()
    redacted: bool = custom_field()


@custom_define()
class DestinyActivitySkullSubcategoryDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        availability_tier_rank: _No description given by bungie._
        default_skull_hashes: _No description given by bungie._
        display_properties: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        parent_skull_category_hash: _No description given by bungie._
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        manifest_parent_skull_category_hash: Manifest information for `parent_skull_category_hash`
    """

    availability_tier_rank: int = custom_field()
    default_skull_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    index: int = custom_field()
    parent_skull_category_hash: int = custom_field()
    redacted: bool = custom_field()
    manifest_parent_skull_category_hash: Optional["DestinyActivitySkullCategoryDefinition"] = custom_field(default=None)


@custom_define()
class DestinyActivityDifficultyTierCollectionDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    None
    Attributes:
        difficulty_tiers: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
    """

    difficulty_tiers: list["DestinyActivityDifficultyTierDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityDifficultyTierDefinition]"""}
    )
    index: int = custom_field()
    redacted: bool = custom_field()


@custom_define()
class DestinyActivityDifficultyTierDefinition(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_level: _No description given by bungie._
        display_properties: _No description given by bungie._
        fixed_activity_skulls: _No description given by bungie._
        maximum_fireteam_leader_power: _No description given by bungie._
        minimum_fireteam_leader_power: _No description given by bungie._
        optional_required_trait: _No description given by bungie._
        recommended_activity_level_offset: _No description given by bungie._
        score_time_limit_multiplier: _No description given by bungie._
        selectable_skull_collection_hashes: _No description given by bungie._
        skull_subcategory_overrides: _No description given by bungie._
        tier_enabled_unlock_expression: _No description given by bungie._
        tier_rank: _No description given by bungie._
        tier_type: _No description given by bungie._
        manifest_optional_required_trait: Manifest information for `optional_required_trait`
    """

    activity_level: int = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    fixed_activity_skulls: list["DestinyActivitySkull"] = custom_field(
        metadata={"type": """list[DestinyActivitySkull]"""}
    )
    maximum_fireteam_leader_power: int = custom_field()
    minimum_fireteam_leader_power: int = custom_field()
    optional_required_trait: int = custom_field()
    recommended_activity_level_offset: int = custom_field()
    score_time_limit_multiplier: int = custom_field()
    selectable_skull_collection_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    skull_subcategory_overrides: list["DestinyActivityDifficultyTierSubcategoryOverride"] = custom_field(
        metadata={"type": """list[DestinyActivityDifficultyTierSubcategoryOverride]"""}
    )
    tier_enabled_unlock_expression: "DestinyUnlockExpressionDefinition" = custom_field()
    tier_rank: int = custom_field()
    tier_type: Union["DestinyActivityDifficultyTierType", int] = custom_field(
        converter=enum_converter("DestinyActivityDifficultyTierType")
    )
    manifest_optional_required_trait: Optional["DestinyTraitDefinition"] = custom_field(default=None)


@custom_define()
class DestinyActivitySkull(BaseModel, HashObject):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_modifier_connotation: _No description given by bungie._
        activity_modifier_display_category: _No description given by bungie._
        display_description_override_for_nav_mode: _No description given by bungie._
        display_in_activity_selection: _No description given by bungie._
        display_in_nav_mode: _No description given by bungie._
        display_properties: _No description given by bungie._
        dynamic_use: _No description given by bungie._
        has_ui: _No description given by bungie._
        hash: _No description given by bungie._
        modifier_multiplier_contribution: _No description given by bungie._
        modifier_power_contribution: _No description given by bungie._
        skull_exclusion_group_hash: _No description given by bungie._
        skull_identifier_hash: _No description given by bungie._
        skull_options: _No description given by bungie._
        manifest_skull_exclusion_group_hash: Manifest information for `skull_exclusion_group_hash`
    """

    activity_modifier_connotation: Union["DestinyActivityModifierConnotation", int] = custom_field(
        converter=enum_converter("DestinyActivityModifierConnotation")
    )
    activity_modifier_display_category: Union["DestinyActivityModifierDisplayCategory", int] = custom_field(
        converter=enum_converter("DestinyActivityModifierDisplayCategory")
    )
    display_description_override_for_nav_mode: str = custom_field()
    display_in_activity_selection: bool = custom_field()
    display_in_nav_mode: bool = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    dynamic_use: Union["DestinyActivitySkullDynamicUse", int] = custom_field(
        converter=enum_converter("DestinyActivitySkullDynamicUse")
    )
    has_ui: bool = custom_field()
    modifier_multiplier_contribution: float = custom_field()
    modifier_power_contribution: int = custom_field()
    skull_exclusion_group_hash: int = custom_field()
    skull_identifier_hash: int = custom_field()
    skull_options: list["DestinyActivitySkullOption"] = custom_field(
        metadata={"type": """list[DestinyActivitySkullOption]"""}
    )
    manifest_skull_exclusion_group_hash: Optional["DestinyActivitySelectableSkullExclusionGroupDefinition"] = (
        custom_field(default=None)
    )


@custom_define()
class DestinyActivitySkullOption(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        bool_value: _No description given by bungie._
        float_value: _No description given by bungie._
        integer_value: _No description given by bungie._
        min_display_difficulty_id: _No description given by bungie._
        option_hash: _No description given by bungie._
        string_value: _No description given by bungie._
    """

    bool_value: bool = custom_field()
    float_value: float = custom_field()
    integer_value: int = custom_field()
    min_display_difficulty_id: Union["DestinyActivityDifficultyId", int] = custom_field(
        converter=enum_converter("DestinyActivityDifficultyId")
    )
    option_hash: int = custom_field()
    string_value: str = custom_field()


@custom_define()
class DestinyActivitySelectableSkullExclusionGroupDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    None
    Attributes:
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
    """

    index: int = custom_field()
    redacted: bool = custom_field()


@custom_define()
class DestinyActivityDifficultyTierSubcategoryOverride(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        refresh_time_minutes: _No description given by bungie._
        refresh_time_offset_minutes: _No description given by bungie._
        skull_subcategory_hash: _No description given by bungie._
    """

    refresh_time_minutes: int = custom_field()
    refresh_time_offset_minutes: int = custom_field()
    skull_subcategory_hash: int = custom_field()


@custom_define()
class DestinyActivitySelectableSkullCollectionDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    None
    Attributes:
        display_properties: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        selectable_activity_skulls: _No description given by bungie._
        selection_type: _No description given by bungie._
        skull_subcategory_hashes: _No description given by bungie._
    """

    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    index: int = custom_field()
    redacted: bool = custom_field()
    selectable_activity_skulls: list["DestinyActivitySelectableSkull"] = custom_field(
        metadata={"type": """list[DestinyActivitySelectableSkull]"""}
    )
    selection_type: "DestinyActivitySelectableSkullCollectionSelectionType" = custom_field()
    skull_subcategory_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})


@custom_define()
class DestinyActivitySelectableSkullCollectionSelectionType(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        refresh_time_minutes: _No description given by bungie._
        refresh_time_offset_minutes: _No description given by bungie._
        selection_count: _No description given by bungie._
    """

    refresh_time_minutes: int = custom_field()
    refresh_time_offset_minutes: int = custom_field()
    selection_count: int = custom_field()


@custom_define()
class DestinyActivitySelectableSkull(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_skull: _No description given by bungie._
        is_empty_skull: _No description given by bungie._
        loadout_restriction_hash: _No description given by bungie._
        required_trait_existence: _No description given by bungie._
        required_trait_hash: _No description given by bungie._
        manifest_loadout_restriction_hash: Manifest information for `loadout_restriction_hash`
        manifest_required_trait_hash: Manifest information for `required_trait_hash`
    """

    activity_skull: "DestinyActivitySkull" = custom_field()
    is_empty_skull: bool = custom_field()
    loadout_restriction_hash: int = custom_field()
    required_trait_existence: bool = custom_field()
    required_trait_hash: int = custom_field()
    manifest_loadout_restriction_hash: Optional["DestinyActivityLoadoutRestrictionDefinition"] = custom_field(
        default=None
    )
    manifest_required_trait_hash: Optional["DestinyTraitDefinition"] = custom_field(default=None)


@custom_define()
class DestinyActivityLoadoutRestrictionDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    None
    Attributes:
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        restricted_equipment_slot_hashes: _No description given by bungie._
        restricted_item_filter_hash: _No description given by bungie._
    """

    index: int = custom_field()
    redacted: bool = custom_field()
    restricted_equipment_slot_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    restricted_item_filter_hash: int = custom_field()


@custom_define()
class DestinyActivityInteractableDefinition(ManifestModel, HashObject):
    """
    There are times in every Activity's life when interacting with an object in the world will result in another Activity activating. Well, not every Activity. Just certain ones. Anyways, this defines a set of interactable components, the activities that they spawn when you interact with them, and the conditions under which they can be interacted with. Sadly, we don't get any *really* good data for them, like positional data... yet. I have hopes for future data that we could put on this.

    None
    Attributes:
        entries: The possible interactables in this activity interactable definition.
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
    """

    entries: list["DestinyActivityInteractableEntryDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityInteractableEntryDefinition]"""}
    )
    index: int = custom_field()
    redacted: bool = custom_field()


@custom_define()
class DestinyActivityInteractableEntryDefinition(BaseModel):
    """
    Defines a specific interactable and the action that can occur when triggered.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: The activity that will trigger when you interact with this interactable.
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_hash: int = custom_field()
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, Union

from bungio.models import CommunityContentSortMode, ForumTopicsCategoryFiltersEnum, PostSearchResponse
from bungio.models.auth import AuthData
from bungio.models.base import ClientMixin, custom_define


@custom_define()
class CommunityContentRouteInterface(ClientMixin):
    async def get_community_content(
        self,
        media_filter: Union[ForumTopicsCategoryFiltersEnum, int],
        page: int,
        sort: Union[CommunityContentSortMode, int],
        auth: Optional[AuthData] = None,
    ) -> PostSearchResponse:
        """
        Returns community content.

        Args:
            media_filter: The type of media to get
            page: Zero based page
            sort: The sort mode.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_community_content(
            media_filter=getattr(media_filter, "value", media_filter),
            page=page,
            sort=getattr(sort, "value", sort),
            auth=auth,
        )
        return await PostSearchResponse.from_dict(
            data=response, client=self._client, media_filter=media_filter, page=page, sort=sort, auth=auth
        )



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, Union

from bungio.models import (
    BungieCredentialType,
    BungieMembershipType,
    GeneralUser,
    GetCredentialTypesForAccountResponse,
    HardLinkedUserMembership,
    UserMembershipData,
    UserSearchPrefixRequest,
    UserSearchResponse,
    UserTheme,
)
from bungio.models.auth import AuthData
from bungio.models.base import ClientMixin, custom_define
from bungio.utils import AllowAsyncIteration


@custom_define()
class UserRouteInterface(ClientMixin):
    async def get_bungie_net_user_by_id(self, id: int, auth: Optional[AuthData] = None) -> GeneralUser:
        """
        Loads a bungienet user by membership id.

        Args:
            id: The requested Bungie.net membership id.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_bungie_net_user_by_id(id=id, auth=auth)
        return await GeneralUser.from_dict(data=response, client=self._client, id=id, auth=auth)

    async def get_sanitized_platform_display_names(
        self, membership_id: int, auth: Optional[AuthData] = None
    ) -> dict[BungieCredentialType, str]:
        """
        Gets a list of all display names linked to this membership id but sanitized (profanity filtered). Obeys all visibility rules of calling user and is heavily cached.

        Args:
            membership_id: The requested membership id to load.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_sanitized_platform_display_names(membership_id=membership_id, auth=auth)
        return {key: value async for key, value in AllowAsyncIteration(response["Response"].items())}

    async def get_credential_types_for_target_account(
        self, membership_id: int, auth: Optional[AuthData] = None
    ) -> list[GetCredentialTypesForAccountResponse]:
        """
        Returns a list of credential types attached to the requested account

        Args:
            membership_id: The user's membership id
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_credential_types_for_target_account(
            membership_id=membership_id, auth=auth
        )
        return [
            await GetCredentialTypesForAccountResponse.from_dict(
                data=value, client=self._client, membership_id=membership_id, auth=auth
            )
            for value in response["Response"]
        ]

    async def get_available_themes(self, auth: Optional[AuthData] = None) -> list[UserTheme]:
        """
        Returns a list of all available user themes.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_available_themes(auth=auth)
        return [await UserTheme.from_dict(data=value, client=self._client, auth=auth) for value in response["Response"]]

    async def get_membership_data_by_id(
        self, membership_id: int, membership_type: Union[BungieMembershipType, int], auth: Optional[AuthData] = None
    ) -> UserMembershipData:
        """
        Returns a list of accounts associated with the supplied membership ID and membership type. This will include all linked accounts (even when hidden) if supplied credentials permit it.

        Args:
            membership_id: The membership ID of the target user.
            membership_type: Type of the supplied membership ID.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_membership_data_by_id(
            membership_id=membership_id, membership_type=getattr(membership_type, "value", membership_type), auth=auth
        )
        return await UserMembershipData.from_dict(
            data=response, client=self._client, membership_id=membership_id, membership_type=membership_type, auth=auth
        )

    async def get_membership_data_for_current_user(self, auth: AuthData) -> UserMembershipData:
        """
        Returns a list of accounts associated with signed in user. This is useful for OAuth implementations that do not give you access to the token response.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadBasicUserProfile

        Args:
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_membership_data_for_current_user(auth=auth)
        return await UserMembershipData.from_dict(data=response, client=self._client, auth=auth)

    async def get_membership_from_hard_linked_credential(
        self, credential: str, cr_type: Union[BungieCredentialType, int], auth: Optional[AuthData] = None
    ) -> HardLinkedUserMembership:
        """
        Gets any hard linked membership given a credential. Only works for credentials that are public (just SteamID64 right now). Cross Save aware.

        Args:
            credential: The credential to look up. Must be a valid SteamID64.
            cr_type: The credential type. 'SteamId' is the only valid value at present.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_membership_from_hard_linked_credential(
            credential=credential, cr_type=getattr(cr_type, "value", cr_type), auth=auth
        )
        return await HardLinkedUserMembership.from_dict(
            data=response, client=self._client, credential=credential, cr_type=cr_type, auth=auth
        )

    async def search_by_global_name_prefix(
        self, display_name_prefix: str, page: int, auth: Optional[AuthData] = None
    ) -> UserSearchResponse:
        """
        [OBSOLETE] Do not use this to search users, use SearchByGlobalNamePost instead.

        Args:
            display_name_prefix: The display name prefix you're looking for.
            page: The zero-based page of results you desire.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.search_by_global_name_prefix(
            display_name_prefix=display_name_prefix, page=page, auth=auth
        )
        return await UserSearchResponse.from_dict(
            data=response, client=self._client, display_name_prefix=display_name_prefix, page=page, auth=auth
        )

    async def search_by_global_name_post(
        self, data: UserSearchPrefixRequest, page: int, auth: Optional[AuthData] = None
    ) -> UserSearchResponse:
        """
        Given the prefix of a global display name, returns all users who share that name.

        Args:
            data: The required data for this request.
            page: The zero-based page of results you desire.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.search_by_global_name_post(
            page=page, auth=auth, **data.to_dict(_return_to_bungie_case=False)
        )
        return await UserSearchResponse.from_dict(data=response, client=self._client, page=page, auth=auth)



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING

from bungio.models.base import BaseModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import DestinyObjectiveProgress


@custom_define()
class DestinyKiosksComponent(BaseModel):
    """
    A Kiosk is a Vendor (DestinyVendorDefinition) that sells items based on whether you have already acquired that item before. This component returns information about what Kiosk items are available to you on a *Profile* level. It is theoretically possible for Kiosks to have items gated by specific Character as well. If you ever have those, you will find them on the individual character's DestinyCharacterKiosksComponent. Note that, because this component returns vendorItemIndexes (that is to say, indexes into the Kiosk Vendor's itemList property), these results are necessarily content version dependent. Make sure that you have the latest version of the content manifest databases before using this data.

    None
    Attributes:
        kiosk_items: A dictionary keyed by the Kiosk Vendor's hash identifier (use it to look up the DestinyVendorDefinition for the relevant kiosk vendor), and whose value is a list of all the items that the user can "see" in the Kiosk, and any other interesting metadata.
    """

    kiosk_items: dict[int, list["DestinyKioskItem"]] = custom_field(
        metadata={"type": """dict[int, list[DestinyKioskItem]]"""}
    )


@custom_define()
class DestinyKioskItem(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        can_acquire: If true, the user can not only see the item, but they can acquire it. It is possible that a user can see a kiosk item and not be able to acquire it.
        failure_indexes: Indexes into failureStrings for the Vendor, indicating the reasons why it failed if any.
        flavor_objective: I may regret naming it this way - but this represents when an item has an objective that doesn't serve a beneficial purpose, but rather is used for "flavor" or additional information. For instance, when Emblems track specific stats, those stats are represented as Objectives on the item.
        index: The index of the item in the related DestinyVendorDefintion's itemList property, representing the sale.
    """

    can_acquire: bool = custom_field()
    failure_indexes: list[int] = custom_field(metadata={"type": """list[int]"""})
    flavor_objective: "DestinyObjectiveProgress" = custom_field()
    index: int = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, Union

from bungio.models import (
    BungieFriendListResponse,
    BungieFriendRequestListResponse,
    PlatformFriendResponse,
    PlatformFriendType,
)
from bungio.models.auth import AuthData
from bungio.models.base import ClientMixin, custom_define


@custom_define()
class SocialRouteInterface(ClientMixin):
    async def get_friend_list(self, auth: AuthData) -> BungieFriendListResponse:
        """
        Returns your Bungie Friend list

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadUserData

        Args:
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_friend_list(auth=auth)
        return await BungieFriendListResponse.from_dict(data=response, client=self._client, auth=auth)

    async def get_friend_request_list(self, auth: AuthData) -> BungieFriendRequestListResponse:
        """
        Returns your friend request queue.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadUserData

        Args:
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_friend_request_list(auth=auth)
        return await BungieFriendRequestListResponse.from_dict(data=response, client=self._client, auth=auth)

    async def issue_friend_request(self, membership_id: str, auth: AuthData) -> bool:
        """
        Requests a friend relationship with the target user. Any of the target user's linked membership ids are valid inputs.

        Warning: Requires Authentication.
            Required oauth2 scopes: BnetWrite

        Args:
            membership_id: The membership id of the user you wish to add.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.issue_friend_request(membership_id=membership_id, auth=auth)
        return response["Response"]

    async def accept_friend_request(self, membership_id: str, auth: AuthData) -> bool:
        """
        Accepts a friend relationship with the target user. The user must be on your incoming friend request list, though no error will occur if they are not.

        Warning: Requires Authentication.
            Required oauth2 scopes: BnetWrite

        Args:
            membership_id: The membership id of the user you wish to accept.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.accept_friend_request(membership_id=membership_id, auth=auth)
        return response["Response"]

    async def decline_friend_request(self, membership_id: str, auth: AuthData) -> bool:
        """
        Declines a friend relationship with the target user. The user must be on your incoming friend request list, though no error will occur if they are not.

        Warning: Requires Authentication.
            Required oauth2 scopes: BnetWrite

        Args:
            membership_id: The membership id of the user you wish to decline.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.decline_friend_request(membership_id=membership_id, auth=auth)
        return response["Response"]

    async def remove_friend(self, membership_id: str, auth: AuthData) -> bool:
        """
        Remove a friend relationship with the target user. The user must be on your friend list, though no error will occur if they are not.

        Warning: Requires Authentication.
            Required oauth2 scopes: BnetWrite

        Args:
            membership_id: The membership id of the user you wish to remove.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.remove_friend(membership_id=membership_id, auth=auth)
        return response["Response"]

    async def remove_friend_request(self, membership_id: str, auth: AuthData) -> bool:
        """
        Remove a friend relationship with the target user. The user must be on your outgoing request friend list, though no error will occur if they are not.

        Warning: Requires Authentication.
            Required oauth2 scopes: BnetWrite

        Args:
            membership_id: The membership id of the user you wish to remove.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.remove_friend_request(membership_id=membership_id, auth=auth)
        return response["Response"]

    async def get_platform_friend_list(
        self, friend_platform: Union[PlatformFriendType, int], page: str, auth: Optional[AuthData] = None
    ) -> PlatformFriendResponse:
        """
        Gets the platform friend of the requested type, with additional information if they have Bungie accounts. Must have a recent login session with said platform.

        Args:
            friend_platform: The platform friend type.
            page: The zero based page to return. Page size is 100.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_platform_friend_list(
            friend_platform=getattr(friend_platform, "value", friend_platform), page=page, auth=auth
        )
        return await PlatformFriendResponse.from_dict(
            data=response, client=self._client, friend_platform=friend_platform, page=page, auth=auth
        )



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Union

from bungio.models.base import BaseEnum, BaseFlagEnum, BaseModel, custom_define, custom_field
from bungio.utils import enum_converter


@custom_define()
class IgnoreResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        ignore_flags: _No description given by bungie._
        is_ignored: _No description given by bungie._
    """

    ignore_flags: Union["IgnoreStatus", int] = custom_field(converter=enum_converter("IgnoreStatus"))
    is_ignored: bool = custom_field()


class IgnoreStatus(BaseFlagEnum):
    """
    _No description given by bungie._
    """

    NOT_IGNORED = 0
    """_No description given by bungie._ """
    IGNORED_USER = 1
    """_No description given by bungie._ """
    IGNORED_GROUP = 2
    """_No description given by bungie._ """
    IGNORED_BY_GROUP = 4
    """_No description given by bungie._ """
    IGNORED_POST = 8
    """_No description given by bungie._ """
    IGNORED_TAG = 16
    """_No description given by bungie._ """
    IGNORED_GLOBAL = 32
    """_No description given by bungie._ """


class IgnoreLength(BaseEnum):
    """
    _No description given by bungie._
    """

    NONE = 0
    """_No description given by bungie._ """
    WEEK = 1
    """_No description given by bungie._ """
    TWO_WEEKS = 2
    """_No description given by bungie._ """
    THREE_WEEKS = 3
    """_No description given by bungie._ """
    MONTH = 4
    """_No description given by bungie._ """
    THREE_MONTHS = 5
    """_No description given by bungie._ """
    SIX_MONTHS = 6
    """_No description given by bungie._ """
    YEAR = 7
    """_No description given by bungie._ """
    FOREVER = 8
    """_No description given by bungie._ """
    THREE_MINUTES = 9
    """_No description given by bungie._ """
    HOUR = 10
    """_No description given by bungie._ """
    THIRTY_DAYS = 11
    """_No description given by bungie._ """



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING

from bungio.models.base import BaseModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import IgnoreResponse


@custom_define()
class TagResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        ignore_status: _No description given by bungie._
        tag_text: _No description given by bungie._
    """

    ignore_status: "IgnoreResponse" = custom_field()
    tag_text: str = custom_field()



from bungio.models.base import custom_define, custom_field
from bungio.models.mixins import DestinyClanMixin

__all__ = ("DestinyClan",)


@custom_define()
class DestinyClan(DestinyClanMixin):
    """
    A representation of a Destiny 2 clan

    Attributes:
        group_id: The clan's id
    """

    group_id: int = custom_field()



from typing import Optional, Type

from bungio.api.bungie import destiny2
from bungio.models.auth import AuthData
from bungio.models.base import ManifestModel, custom_define


@custom_define()
class Destiny2RouteInterface(destiny2.Destiny2RouteInterface):
    async def get_destiny_entity_definition(
        self, entity_type: Type[ManifestModel], hash_identifier: int, auth: Optional[AuthData] = None
    ) -> ManifestModel:
        """
        Returns the static definition of an entity of the given Type and hash identifier. Examine the API Documentation for the Type Names of entities that
        have their own definitions. Note that the return type will always *inherit from* DestinyDefinition, but the specific type returned will be the
        requested entity type if it can be found. Please don't use this as a chatty alternative to the Manifest database if you require large sets of data,
        but for simple and one-off accesses this should be handy.

        Args:
            entity_type: The type of entity for whom you would like results. These correspond to the entity's definition contract name. For instance, if you are looking for items, this property should be 'DestinyInventoryItemDefinition'. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is tentatively in final form, but there may be bugs that prevent desirable operation.
            hash_identifier: The hash identifier for the specific Entity you want returned.
            auth: Authentication information. Required when users with a private profile are queried.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_destiny_entity_definition(
            entity_type=entity_type.__name__, hash_identifier=hash_identifier, auth=auth
        )
        return await entity_type.from_dict(data=response, client=self._client)



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING

from bungio.models.base import BaseModel, HashObject, ManifestModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import DestinyDisplayPropertiesDefinition


@custom_define()
class DestinyReportReasonCategoryDefinition(ManifestModel, HashObject):
    """
    If you're going to report someone for a Terms of Service violation, you need to choose a category and reason for the report. This definition holds both the categories and the reasons within those categories, for simplicity and my own laziness' sake. Note tha this means that, to refer to a Reason by reasonHash, you need a combination of the reasonHash *and* the associated ReasonCategory's hash: there are some reasons defined under multiple categories.

    None
    Attributes:
        display_properties: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        reasons: The specific reasons for the report under this category.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
    """

    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    index: int = custom_field()
    reasons: dict[int, "DestinyReportReasonDefinition"] = custom_field(
        metadata={"type": """dict[int, DestinyReportReasonDefinition]"""}
    )
    redacted: bool = custom_field()


@custom_define()
class DestinyReportReasonDefinition(BaseModel):
    """
    A specific reason for being banned. Only accessible under the related category (DestinyReportReasonCategoryDefinition) under which it is shown. Note that this means that report reasons' reasonHash are not globally unique: and indeed, entries like "Other" are defined under most categories for example.

    None
    Attributes:
        display_properties: _No description given by bungie._
        reason_hash: The identifier for the reason: they are only guaranteed unique under the Category in which they are found.
    """

    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    reason_hash: int = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import BaseModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import (
        DestinyEventCardDefinition,
        DestinyGameVersions,
        DestinyGuardianRankDefinition,
        DestinySeasonDefinition,
        DestinySeasonPassDefinition,
        DestinyVendorReceipt,
        UserInfoCard,
    )


@custom_define()
class DestinyVendorReceiptsComponent(BaseModel):
    """
    For now, this isn't used for much: it's a record of the recent refundable purchases that the user has made. In the future, it could be used for providing refunds/buyback via the API. Wouldn't that be fun?

    None
    Attributes:
        receipts: The receipts for refundable purchases made at a vendor.
    """

    receipts: list["DestinyVendorReceipt"] = custom_field(metadata={"type": """list[DestinyVendorReceipt]"""})


@custom_define()
class DestinyProfileComponent(BaseModel):
    """
    The most essential summary information about a Profile (in Destiny 1, we called these "Accounts").

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        active_event_card_hash: If populated, this is a reference to the event card that is currently active.
        character_ids: A list of the character IDs, for further querying on your part.
        current_guardian_rank: The 'current' Guardian Rank value, which starts at rank 1. This rank value will drop at the start of a new season to your 'renewed' rank from the previous season.
        current_season_hash: If populated, this is a reference to the season that is currently active.
        current_season_pass_hash: If populated, this is a reference to the season pass that is currently active.
        current_season_reward_power_cap: If populated, this is the reward power cap for the current season.
        date_last_played: The last time the user played with any character on this Profile.
        event_card_hashes_owned: A list of hashes for event cards that a profile owns. Unlike most values in versionsOwned, these stay with the profile across all platforms.
        lifetime_highest_guardian_rank: The 'lifetime highest' Guardian Rank value, which starts at rank 1. This rank value should never go down.
        renewed_guardian_rank: The seasonal 'renewed' Guardian Rank value. This rank value resets at the start of each new season to the highest-earned non-advanced rank.
        season_hashes: A list of seasons that this profile owns. Unlike versionsOwned, these stay with the profile across Platforms, and thus will be valid.  It turns out that Stadia Pro subscriptions will give access to seasons but only while playing on Stadia and with an active subscription. So some users (users who have Stadia Pro but choose to play on some other platform) won't see these as available: it will be whatever seasons are available for the platform on which they last played.
        season_pass_hashes: A list of season passes aka reward passes that this profile owns. Unlike versionsOwned, these stay with the profile across Platforms, and thus will be valid.
        user_info: If you need to render the Profile (their platform name, icon, etc...) somewhere, this property contains that information.
        versions_owned: If you want to know what expansions they own, this will contain that data.  IMPORTANT: This field may not return the data you're interested in for Cross-Saved users. It returns the last ownership data we saw for this account - which is to say, what they've purchased on the platform on which they last played, which now could be a different platform.  If you don't care about per-platform ownership and only care about whatever platform it seems they are playing on most recently, then this should be "good enough." Otherwise, this should be considered deprecated. We do not have a good alternative to provide at this time with platform specific ownership data for DLC.
        manifest_active_event_card_hash: Manifest information for `active_event_card_hash`
        manifest_current_guardian_rank: Manifest information for `current_guardian_rank`
        manifest_current_season_hash: Manifest information for `current_season_hash`
        manifest_current_season_pass_hash: Manifest information for `current_season_pass_hash`
        manifest_lifetime_highest_guardian_rank: Manifest information for `lifetime_highest_guardian_rank`
        manifest_renewed_guardian_rank: Manifest information for `renewed_guardian_rank`
    """

    active_event_card_hash: int = custom_field()
    character_ids: list[int] = custom_field(metadata={"type": """list[int]"""})
    current_guardian_rank: int = custom_field()
    current_season_hash: int = custom_field()
    current_season_pass_hash: int = custom_field()
    current_season_reward_power_cap: int = custom_field()
    date_last_played: datetime = custom_field()
    event_card_hashes_owned: list[int] = custom_field(metadata={"type": """list[int]"""})
    lifetime_highest_guardian_rank: int = custom_field()
    renewed_guardian_rank: int = custom_field()
    season_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    season_pass_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    user_info: "UserInfoCard" = custom_field()
    versions_owned: Union["DestinyGameVersions", int] = custom_field(converter=enum_converter("DestinyGameVersions"))
    manifest_active_event_card_hash: Optional["DestinyEventCardDefinition"] = custom_field(default=None)
    manifest_current_guardian_rank: Optional["DestinyGuardianRankDefinition"] = custom_field(default=None)
    manifest_current_season_hash: Optional["DestinySeasonDefinition"] = custom_field(default=None)
    manifest_current_season_pass_hash: Optional["DestinySeasonPassDefinition"] = custom_field(default=None)
    manifest_lifetime_highest_guardian_rank: Optional["DestinyGuardianRankDefinition"] = custom_field(default=None)
    manifest_renewed_guardian_rank: Optional["DestinyGuardianRankDefinition"] = custom_field(default=None)



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import BaseEnum, BaseModel, HashObject, ManifestModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import (
        DestinyActivityDefinition,
        DestinyDestinationDefinition,
        DestinyDisplayPropertiesDefinition,
        DestinyInventoryItemDefinition,
        DestinyItemQuantity,
        DestinyObjectiveDefinition,
        DestinyVendorDefinition,
    )


@custom_define()
class DestinyMilestoneDefinition(ManifestModel, HashObject):
    """
    Milestones are an in-game concept where they're attempting to tell you what you can do next in-game. If that sounds a lot like Advisors in Destiny 1, it is! So we threw out Advisors in the Destiny 2 API and tacked all of the data we would have put on Advisors onto Milestones instead. Each Milestone represents something going on in the game right now: - A "ritual activity" you can perform, like nightfall - A "special event" that may have activities related to it, like Taco Tuesday (there's no Taco Tuesday in Destiny 2) - A checklist you can fulfill, like helping your Clan complete all of its weekly objectives - A tutorial quest you can play through, like the introduction to the Crucible. Most of these milestones appear in game as well. Some of them are BNet only, because we're so extra. You're welcome. There are some important caveats to understand about how we currently render Milestones and their deficiencies. The game currently doesn't have any content that actually tells you oughtright *what* the Milestone is: that is to say, what you'll be doing. The best we get is either a description of the overall Milestone, or of the Quest that the Milestone is having you partake in: which is usually something that assumes you already know what it's talking about, like "Complete 5 Challenges". 5 Challenges for what? What's a challenge? These are not questions that the Milestone data will answer for you unfortunately. This isn't great, and in the future I'd like to add some custom text to give you more contextual information to pass on to your users. But for now, you can do what we do to render what little display info we do have: Start by looking at the currently active quest (ideally, you've fetched DestinyMilestone or DestinyPublicMilestone data from the API, so you know the currently active quest for the Milestone in question). Look up the Quests property in the Milestone Definition, and check if it has display properties. If it does, show that as the description of the Milestone. If it doesn't, fall back on the Milestone's description. This approach will let you avoid, whenever possible, the even less useful (and sometimes nonexistant) milestone-level names and descriptions.

    None
    Attributes:
        activities: A Milestone can now be represented by one or more activities directly (without a backing Quest), and that activity can have many challenges, modifiers, and related to it.
        default_order: _No description given by bungie._
        display_preference: A hint to the UI to indicate what to show as the display properties for this Milestone when showing "Live" milestone data. Feel free to show more than this if desired: this hint is meant to simplify our own UI, but it may prove useful to you as well.
        display_properties: _No description given by bungie._
        explore_prioritizes_activity_image: If TRUE, "Explore Destiny" (the front page of BNet and the companion app) prioritize using the activity image over any overriding Quest or Milestone image provided. This unfortunate hack is brought to you by Trials of The Nine.
        friendly_name: If the milestone has a friendly identifier for association with other features - such as Recruiting - that identifier can be found here. This is "friendly" in that it looks better in a URL than whatever the identifier for the Milestone actually is.
        has_predictable_dates: A shortcut for clients - and the server - to understand whether we can predict the start and end dates for this event. In practice, there are multiple ways that an event could have predictable date ranges, but not all events will be able to be predicted via any mechanism (for instance, events that are manually triggered on and off)
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        image: A custom image someone made just for the milestone. Isn't that special?
        index: The index of the entity as it was found in the investment tables.
        is_in_game_milestone: Some milestones are explicit objectives that you can see and interact with in the game. Some milestones are more conceptual, built by BNet to help advise you on activities and events that happen in-game but that aren't explicitly shown in game as Milestones. If this is TRUE, you can see this as a milestone in the game. If this is FALSE, it's an event or activity you can participate in, but you won't see it as a Milestone in the game's UI.
        milestone_type: An enumeration listing one of the possible types of milestones. Check out the DestinyMilestoneType enum for more info!
        quests: The full set of possible Quests that give the overview of the Milestone event/activity in question. Only one of these can be active at a time for a given Conceptual Milestone, but many of them may be "available" for the user to choose from. (for instance, with Milestones you can choose from the three available Quests, but only one can be active at a time) Keyed by the quest item. As of Forsaken (~September 2018), Quest-style Milestones are being removed for many types of activities. There will likely be further revisions to the Milestone concept in the future.
        recruitable: If True, then the Milestone has been integrated with BNet's recruiting feature.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        rewards: If this milestone can provide rewards, this will define the categories into which the individual reward entries are placed. This is keyed by the Category's hash, which is only guaranteed to be unique within a given Milestone.
        show_in_explorer: If TRUE, this entry should be returned in the list of milestones for the "Explore Destiny" (i.e. new BNet homepage) features of Bungie.net (as long as the underlying event is active) Note that this is a property specifically used by BNet and the companion app for the "Live Events" feature of the front page/welcome view: it's not a reflection of what you see in-game.
        show_in_milestones: Determines whether we'll show this Milestone in the user's personal Milestones list.
        values: Sometimes, milestones will have arbitrary values associated with them that are of interest to us or to third party developers. This is the collection of those values' definitions, keyed by the identifier of the value and providing useful definition information such as localizable names and descriptions for the value.
        vendors: Sometimes, milestones will have rewards provided by Vendors. This definition gives the information needed to understand which vendors are relevant, the order in which they should be returned if order matters, and the conditions under which the Vendor is relevant to the user.
        vendors_display_title: If you're going to show Vendors for the Milestone, you can use this as a localized "header" for the section where you show that vendor data. It'll provide a more context-relevant clue about what the vendor's role is in the Milestone.
    """

    activities: list["DestinyMilestoneChallengeActivityDefinition"] = custom_field(
        metadata={"type": """list[DestinyMilestoneChallengeActivityDefinition]"""}
    )
    default_order: int = custom_field()
    display_preference: Union["DestinyMilestoneDisplayPreference", int] = custom_field(
        converter=enum_converter("DestinyMilestoneDisplayPreference")
    )
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    explore_prioritizes_activity_image: bool = custom_field()
    friendly_name: str = custom_field()
    has_predictable_dates: bool = custom_field()
    image: str = custom_field()
    index: int = custom_field()
    is_in_game_milestone: bool = custom_field()
    milestone_type: Union["DestinyMilestoneType", int] = custom_field(converter=enum_converter("DestinyMilestoneType"))
    quests: dict[int, "DestinyMilestoneQuestDefinition"] = custom_field(
        metadata={"type": """dict[int, DestinyMilestoneQuestDefinition]"""}
    )
    recruitable: bool = custom_field()
    redacted: bool = custom_field()
    rewards: dict[int, "DestinyMilestoneRewardCategoryDefinition"] = custom_field(
        metadata={"type": """dict[int, DestinyMilestoneRewardCategoryDefinition]"""}
    )
    show_in_explorer: bool = custom_field()
    show_in_milestones: bool = custom_field()
    values: dict[str, "DestinyMilestoneValueDefinition"] = custom_field(
        metadata={"type": """dict[str, DestinyMilestoneValueDefinition]"""}
    )
    vendors: list["DestinyMilestoneVendorDefinition"] = custom_field(
        metadata={"type": """list[DestinyMilestoneVendorDefinition]"""}
    )
    vendors_display_title: str = custom_field()


class DestinyMilestoneDisplayPreference(BaseEnum):
    """
    A hint for the UI as to what display information ought to be shown. Defaults to showing the static MilestoneDefinition's display properties.  If for some reason the indicated property is not populated, fall back to the MilestoneDefinition.displayProperties.
    """

    MILESTONE_DEFINITION = 0
    """Indicates you should show DestinyMilestoneDefinition.displayProperties for this Milestone. """
    CURRENT_QUEST_STEPS = 1
    """Indicates you should show the displayProperties for any currently active Quest Steps in DestinyMilestone.availableQuests. """
    CURRENT_ACTIVITY_CHALLENGES = 2
    """Indicates you should show the displayProperties for any currently active Activities and their Challenges in DestinyMilestone.activities. """


class DestinyMilestoneType(BaseEnum):
    """
    The type of milestone. Milestones can be Tutorials, one-time/triggered/non-repeating but not necessarily tutorials, or Repeating Milestones.
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    TUTORIAL = 1
    """One-time milestones that are specifically oriented toward teaching players about new mechanics and gameplay modes. """
    ONE_TIME = 2
    """Milestones that, once completed a single time, can never be repeated. """
    WEEKLY = 3
    """Milestones that repeat/reset on a weekly basis. They need not all reset on the same day or time, but do need to reset weekly to qualify for this type. """
    DAILY = 4
    """Milestones that repeat or reset on a daily basis. """
    SPECIAL = 5
    """Special indicates that the event is not on a daily/weekly cadence, but does occur more than once. For instance, Iron Banner in Destiny 1 or the Dawning were examples of what could be termed "Special" events. """


@custom_define()
class DestinyMilestoneQuestDefinition(BaseModel):
    """
    Any data we need to figure out whether this Quest Item is the currently active one for the conceptual Milestone. Even just typing this description, I already regret it.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activities: The full set of all possible "conceptual activities" that are related to this Milestone. Tiers or alternative modes of play within these conceptual activities will be defined as sub-entities. Keyed by the Conceptual Activity Hash. Use the key to look up DestinyActivityDefinition.
        destination_hash: Sometimes, a Milestone's quest is related to an entire Destination rather than a specific activity. In that situation, this will be the hash of that Destination. Hotspots are currently the only Milestones that expose this data, but that does not preclude this data from being returned for other Milestones in the future.
        display_properties: The individual quests may have different definitions from the overall milestone: if there's a specific active quest, use these displayProperties instead of that of the overall DestinyMilestoneDefinition.
        override_image: If populated, this image can be shown instead of the generic milestone's image when this quest is live, or it can be used to show a background image for the quest itself that differs from that of the Activity or the Milestone.
        quest_item_hash: The item representing this Milestone quest. Use this hash to look up the DestinyInventoryItemDefinition for the quest to find its steps and human readable data.
        quest_rewards: The rewards you will get for completing this quest, as best as we could extract them from our data. Sometimes, it'll be a decent amount of data. Sometimes, it's going to be sucky. Sorry.
        manifest_destination_hash: Manifest information for `destination_hash`
        manifest_quest_item_hash: Manifest information for `quest_item_hash`
    """

    activities: dict[int, "DestinyMilestoneActivityDefinition"] = custom_field(
        metadata={"type": """dict[int, DestinyMilestoneActivityDefinition]"""}
    )
    destination_hash: int = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    override_image: str = custom_field()
    quest_item_hash: int = custom_field()
    quest_rewards: "DestinyMilestoneQuestRewardsDefinition" = custom_field()
    manifest_destination_hash: Optional["DestinyDestinationDefinition"] = custom_field(default=None)
    manifest_quest_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneQuestRewardsDefinition(BaseModel):
    """
    If rewards are given in a quest - as opposed to overall in the entire Milestone - there's way less to track. We're going to simplify this contract as a result. However, this also gives us the opportunity to potentially put more than just item information into the reward data if we're able to mine it out in the future. Remember this if you come back and ask "why are quest reward items nested inside of their own class?"

    None
    Attributes:
        items: The items that represent your reward for completing the quest. Be warned, these could be "dummy" items: items that are only used to render a good-looking in-game tooltip, but aren't the actual items themselves. For instance, when experience is given there's often a dummy item representing "experience", with quantity being the amount of experience you got. We don't have a programmatic association between those and whatever Progression is actually getting that experience... yet.
    """

    items: list["DestinyMilestoneQuestRewardItem"] = custom_field(
        metadata={"type": """list[DestinyMilestoneQuestRewardItem]"""}
    )


@custom_define()
class DestinyMilestoneQuestRewardItem(BaseModel):
    """
    A subclass of DestinyItemQuantity, that provides not just the item and its quantity but also information that BNet can - at some point - use internally to provide more robust runtime information about the item's qualities. If you want it, please ask! We're just out of time to wire it up right now. Or a clever person just may do it with our existing endpoints.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        has_conditional_visibility: Indicates that this item quantity may be conditionally shown or hidden, based on various sources of state. For example: server flags, account state, or character progress.
        item_hash: The hash identifier for the item in question. Use it to look up the item's DestinyInventoryItemDefinition.
        item_instance_id: If this quantity is referring to a specific instance of an item, this will have the item's instance ID. Normally, this will be null.
        quantity: The amount of the item needed/available depending on the context of where DestinyItemQuantity is being used.
        vendor_hash: The quest reward item *may* be associated with a vendor. If so, this is that vendor. Use this hash to look up the DestinyVendorDefinition.
        vendor_item_index: The quest reward item *may* be associated with a vendor. If so, this is the index of the item being sold, which we can use at runtime to find instanced item information for the reward item.
        manifest_item_hash: Manifest information for `item_hash`
        manifest_vendor_hash: Manifest information for `vendor_hash`
    """

    has_conditional_visibility: bool = custom_field()
    item_hash: int = custom_field()
    item_instance_id: int = custom_field(metadata={"int64": True})
    quantity: int = custom_field()
    vendor_hash: int = custom_field()
    vendor_item_index: int = custom_field()
    manifest_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneActivityDefinition(BaseModel):
    """
    Milestones can have associated activities which provide additional information about the context, challenges, modifiers, state etc... related to this Milestone.  Information we need to be able to return that data is defined here, along with Tier data to establish a relationship between a conceptual Activity and its difficulty levels and variants.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        conceptual_activity_hash: The "Conceptual" activity hash. Basically, we picked the lowest level activity and are treating it as the canonical definition of the activity for rendering purposes. If you care about the specific difficulty modes and variations, use the activities under "Variants".
        variants: A milestone-referenced activity can have many variants, such as Tiers or alternative modes of play. Even if there is only a single variant, the details for these are represented within as a variant definition. It is assumed that, if this DestinyMilestoneActivityDefinition is active, then all variants should be active. If a Milestone could ever split the variants' active status conditionally, they should all have their own DestinyMilestoneActivityDefinition instead! The potential duplication will be worth it for the obviousness of processing and use.
        manifest_conceptual_activity_hash: Manifest information for `conceptual_activity_hash`
    """

    conceptual_activity_hash: int = custom_field()
    variants: dict[int, "DestinyMilestoneActivityVariantDefinition"] = custom_field(
        metadata={"type": """dict[int, DestinyMilestoneActivityVariantDefinition]"""}
    )
    manifest_conceptual_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneActivityVariantDefinition(BaseModel):
    """
    Represents a variant on an activity for a Milestone: a specific difficulty tier, or a specific activity variant for example. These will often have more specific details, such as an associated Guided Game, progression steps, tier-specific rewards, and custom values.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: The hash to use for looking up the variant Activity's definition (DestinyActivityDefinition), where you can find its distinguishing characteristics such as difficulty level and recommended light level.  Frequently, that will be the only distinguishing characteristics in practice, which is somewhat of a bummer.
        order: If you care to do so, render the variants in the order prescribed by this value. When you combine live Milestone data with the definition, the order becomes more useful because you'll be cross-referencing between the definition and live data.
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_hash: int = custom_field()
    order: int = custom_field()
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneRewardCategoryDefinition(BaseModel):
    """
    The definition of a category of rewards, that contains many individual rewards.

    None
    Attributes:
        category_hash: Identifies the reward category. Only guaranteed unique within this specific component!
        category_identifier: The string identifier for the category, if you want to use it for some end. Guaranteed unique within the specific component.
        display_properties: Hopefully this is obvious by now.
        order: If you want to use BNet's recommended order for rendering categories programmatically, use this value and compare it to other categories to determine the order in which they should be rendered. I don't feel great about putting this here, I won't lie.
        reward_entries: If this milestone can provide rewards, this will define the sets of rewards that can be earned, the conditions under which they can be acquired, internal data that we'll use at runtime to determine whether you've already earned or redeemed this set of rewards, and the category that this reward should be placed under.
    """

    category_hash: int = custom_field()
    category_identifier: str = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    order: int = custom_field()
    reward_entries: dict[int, "DestinyMilestoneRewardEntryDefinition"] = custom_field(
        metadata={"type": """dict[int, DestinyMilestoneRewardEntryDefinition]"""}
    )


@custom_define()
class DestinyMilestoneRewardEntryDefinition(BaseModel):
    """
    The definition of a specific reward, which may be contained in a category of rewards and that has optional information about how it is obtained.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        display_properties: For us to bother returning this info, we should be able to return some kind of information about why these rewards are grouped together. This is ideally that information. Look at how confident I am that this will always remain true.
        items: The items you will get as rewards, and how much of it you'll get.
        order: If you want to follow BNet's ordering of these rewards, use this number within a given category to order the rewards. Yeah, I know. I feel dirty too.
        reward_entry_hash: The identifier for this reward entry. Runtime data will refer to reward entries by this hash. Only guaranteed unique within the specific Milestone.
        reward_entry_identifier: The string identifier, if you care about it. Only guaranteed unique within the specific Milestone.
        vendor_hash: If this reward is redeemed at a Vendor, this is the hash of the Vendor to go to in order to redeem the reward. Use this hash to look up the DestinyVendorDefinition.
        manifest_vendor_hash: Manifest information for `vendor_hash`
    """

    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    items: list["DestinyItemQuantity"] = custom_field(metadata={"type": """list[DestinyItemQuantity]"""})
    order: int = custom_field()
    reward_entry_hash: int = custom_field()
    reward_entry_identifier: str = custom_field()
    vendor_hash: int = custom_field()
    manifest_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneVendorDefinition(BaseModel):
    """
    If the Milestone or a component has vendors whose inventories could/should be displayed that are relevant to it, this will return the vendor in question.  It also contains information we need to determine whether that vendor is actually relevant at the moment, given the user's current state.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        vendor_hash: The hash of the vendor whose wares should be shown as associated with the Milestone.
        manifest_vendor_hash: Manifest information for `vendor_hash`
    """

    vendor_hash: int = custom_field()
    manifest_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneValueDefinition(BaseModel):
    """
    The definition for information related to a key/value pair that is relevant for a particular Milestone or component within the Milestone.  This lets us more flexibly pass up information that's useful to someone, even if it's not necessarily us.

    None
    Attributes:
        display_properties: _No description given by bungie._
        key: _No description given by bungie._
    """

    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    key: str = custom_field()


@custom_define()
class DestinyMilestoneChallengeActivityDefinition(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_graph_nodes: If the activity and its challenge is visible on any of these nodes, it will be returned.
        activity_hash: The activity for which this challenge is active.
        challenges: _No description given by bungie._
        phases: Phases related to this activity, if there are any. These will be listed in the order in which they will appear in the actual activity.
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_graph_nodes: list["DestinyMilestoneChallengeActivityGraphNodeEntry"] = custom_field(
        metadata={"type": """list[DestinyMilestoneChallengeActivityGraphNodeEntry]"""}
    )
    activity_hash: int = custom_field()
    challenges: list["DestinyMilestoneChallengeDefinition"] = custom_field(
        metadata={"type": """list[DestinyMilestoneChallengeDefinition]"""}
    )
    phases: list["DestinyMilestoneChallengeActivityPhase"] = custom_field(
        metadata={"type": """list[DestinyMilestoneChallengeActivityPhase]"""}
    )
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneChallengeDefinition(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        challenge_objective_hash: The challenge related to this milestone.
        manifest_challenge_objective_hash: Manifest information for `challenge_objective_hash`
    """

    challenge_objective_hash: int = custom_field()
    manifest_challenge_objective_hash: Optional["DestinyObjectiveDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneChallengeActivityGraphNodeEntry(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        activity_graph_hash: _No description given by bungie._
        activity_graph_node_hash: _No description given by bungie._
    """

    activity_graph_hash: int = custom_field()
    activity_graph_node_hash: int = custom_field()


@custom_define()
class DestinyMilestoneChallengeActivityPhase(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        phase_hash: The hash identifier of the activity's phase.
    """

    phase_hash: int = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import HashObject, ManifestModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import DestinyDisplayPropertiesDefinition, DestinyEnergyType, DestinyStatDefinition


@custom_define()
class DestinyEnergyTypeDefinition(ManifestModel, HashObject):
    """
    Represents types of Energy that can be used for costs and payments related to Armor 2.0 mods.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        capacity_stat_hash: If this Energy Type can be used for determining the Type of Energy that an item can consume, this is the hash for the DestinyInvestmentStatDefinition that represents the stat which holds the Capacity for that energy type. (Note that this is optional because "Any" is a valid cost, but not valid for Capacity - an Armor must have a specific Energy Type for determining the energy type that the Armor is restricted to use)
        cost_stat_hash: If this Energy Type can be used as a cost to pay for socketing Armor 2.0 items, this is the hash for the DestinyInvestmentStatDefinition that stores the plug's raw cost.
        display_properties: The description of the energy type, icon etc...
        enum_value: We have an enumeration for Energy types for quick reference. This is the current definition's Energy type enum value.
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        show_icon: If TRUE, the game shows this Energy type's icon. Otherwise, it doesn't. Whether you show it or not is up to you.
        transparent_icon_path: A variant of the icon that is transparent and colorless.
        manifest_capacity_stat_hash: Manifest information for `capacity_stat_hash`
        manifest_cost_stat_hash: Manifest information for `cost_stat_hash`
    """

    capacity_stat_hash: int = custom_field()
    cost_stat_hash: int = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    enum_value: Union["DestinyEnergyType", int] = custom_field(converter=enum_converter("DestinyEnergyType"))
    index: int = custom_field()
    redacted: bool = custom_field()
    show_icon: bool = custom_field()
    transparent_icon_path: str = custom_field()
    manifest_capacity_stat_hash: Optional["DestinyStatDefinition"] = custom_field(default=None)
    manifest_cost_stat_hash: Optional["DestinyStatDefinition"] = custom_field(default=None)



from enum import Enum

__all__ = ("BungieLanguage",)


class BungieLanguage(Enum):
    """
    Set your preferred language. This will impact the language of the manifest
    """

    ENGLISH = "en"
    FRENCH = "fr"
    SPANISH = "es"
    MEXICAN = "es-mx"
    GERMAN = "de"
    ITALIAN = "it"
    JAPANESE = "ja"
    PORTUGUESE = "pt-br"
    RUSSIAN = "ru"
    POLISH = "pl"
    KOREAN = "ko"
    CHINESE_TRADITIONAL = "zh-cht"
    CHINESE_SIMPLIFIED = "zh-chs"



from typing import TYPE_CHECKING

from bungio.models.base import custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models.auth import AuthData

__all__ = (
    "BungIOException",
    "HttpException",
    "BungieException",
    "InvalidAuthentication",
    "NotFound",
    "BadRequest",
    "AuthenticationTooSlow",
    "BungieDead",
    "TimeoutException",
)


@custom_define()
class BungIOException(Exception):
    """
    The base exception.
    """

    pass


@custom_define()
class HttpException(BungIOException):
    """
    The base http exception.
    """

    pass


@custom_define()
class BungieException(HttpException):
    """
    An exception raised by bungie

    Attributes:
        error: The exception bungie raised
        message: The message bungie returned
        code: The code of the exception
        data: The data of the exception
    """

    error: str = custom_field()
    message: str = custom_field()
    code: int = custom_field()
    data: dict = custom_field()


@custom_define()
class _InvalidAuthentication(HttpException):
    pass


@custom_define()
class InvalidAuthentication(HttpException):
    """
    Raised if the passed authentication is invalid

    Attributes:
        auth: The passed authentication
    """

    auth: "AuthData"


@custom_define()
class NotFound(HttpException):
    """
    Raised if the resource was not found (404)
    """

    pass


@custom_define()
class BadRequest(HttpException):
    """
    Raised if the resource returned a generic error (400)
    """

    pass


@custom_define()
class AuthenticationTooSlow(HttpException):
    """
    Raised if the authentication grant expired
    """

    pass


@custom_define()
class BungieDead(HttpException):
    """
    Raised if bungie is down. Usually this mean maintenance, see their [twitter](https://twitter.com/BungieHelp) for more info
    """

    pass


@custom_define()
class _RouteError(HttpException):
    route: str


@custom_define()
class TimeoutException(HttpException):
    """
    If the requests failed multiple times in a row without a specific exception
    """

    pass



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.


from bungio.models.base import BaseModel, custom_define, custom_field


@custom_define()
class DestinySocialCommendationsComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        commendation_node_percentages_by_hash: The percentage for each commendation type out of total received
        commendation_node_scores_by_hash: _No description given by bungie._
        commendation_scores_by_hash: _No description given by bungie._
        score_detail_values: _No description given by bungie._
        total_score: _No description given by bungie._
    """

    commendation_node_percentages_by_hash: dict[int, int] = custom_field(metadata={"type": """dict[int, int]"""})
    commendation_node_scores_by_hash: dict[int, int] = custom_field(metadata={"type": """dict[int, int]"""})
    commendation_scores_by_hash: dict[int, int] = custom_field(metadata={"type": """dict[int, int]"""})
    score_detail_values: list[int] = custom_field(metadata={"type": """list[int]"""})
    total_score: int = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import BaseModel, HashObject, ManifestModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import (
        DestinyDisplayPropertiesDefinition,
        DestinyGender,
        DestinyItemQuantity,
        DestinyLoreDefinition,
        DestinyObjectiveDefinition,
        DestinyPresentationChildBlock,
        DestinyPresentationNodeRequirementsBlock,
        DestinyPresentationNodeType,
        DestinyRecordToastStyle,
        DestinyRecordValueStyle,
        DestinyScope,
    )


@custom_define()
class DestinyRecordDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        completion_info: _No description given by bungie._
        display_properties: _No description given by bungie._
        expiration_info: _No description given by bungie._
        for_title_gilding: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        interval_info: Some records have multiple 'interval' objectives, and the record may be claimed at each completed interval
        lore_hash: _No description given by bungie._
        objective_hashes: _No description given by bungie._
        parent_node_hashes: A quick reference to presentation nodes that have this node as a child. Presentation nodes can be parented under multiple parents.
        presentation_info: _No description given by bungie._
        presentation_node_type: _No description given by bungie._
        record_type_name: A display name for the type of record this is (Triumphs, Lore, Medals, Seasonal Challenge, etc.).
        record_value_style: _No description given by bungie._
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        requirements: _No description given by bungie._
        reward_items: If there is any publicly available information about rewards earned for achieving this record, this is the list of those items.  However, note that some records intentionally have "hidden" rewards. These will not be returned in this list.
        scope: Indicates whether this Record's state is determined on a per-character or on an account-wide basis.
        should_show_large_icons: A hint to show a large icon for a reward
        state_info: _No description given by bungie._
        title_info: _No description given by bungie._
        trait_hashes: _No description given by bungie._
        trait_ids: _No description given by bungie._
        manifest_lore_hash: Manifest information for `lore_hash`
    """

    completion_info: "DestinyRecordCompletionBlock" = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    expiration_info: "DestinyRecordExpirationBlock" = custom_field()
    for_title_gilding: bool = custom_field()
    index: int = custom_field()
    interval_info: "DestinyRecordIntervalBlock" = custom_field()
    lore_hash: int = custom_field()
    objective_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    parent_node_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    presentation_info: "DestinyPresentationChildBlock" = custom_field()
    presentation_node_type: Union["DestinyPresentationNodeType", int] = custom_field(
        converter=enum_converter("DestinyPresentationNodeType")
    )
    record_type_name: str = custom_field()
    record_value_style: Union["DestinyRecordValueStyle", int] = custom_field(
        converter=enum_converter("DestinyRecordValueStyle")
    )
    redacted: bool = custom_field()
    requirements: "DestinyPresentationNodeRequirementsBlock" = custom_field()
    reward_items: list["DestinyItemQuantity"] = custom_field(metadata={"type": """list[DestinyItemQuantity]"""})
    scope: Union["DestinyScope", int] = custom_field(converter=enum_converter("DestinyScope"))
    should_show_large_icons: bool = custom_field()
    state_info: "SchemaRecordStateBlock" = custom_field()
    title_info: "DestinyRecordTitleBlock" = custom_field()
    trait_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    trait_ids: list[str] = custom_field(metadata={"type": """list[str]"""})
    manifest_lore_hash: Optional["DestinyLoreDefinition"] = custom_field(default=None)


@custom_define()
class DestinyRecordTitleBlock(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        gilding_tracking_record_hash: _No description given by bungie._
        has_title: _No description given by bungie._
        titles_by_gender: _No description given by bungie._
        titles_by_gender_hash: For those who prefer to use the definitions.
        manifest_gilding_tracking_record_hash: Manifest information for `gilding_tracking_record_hash`
    """

    gilding_tracking_record_hash: int = custom_field()
    has_title: bool = custom_field()
    titles_by_gender: dict[Union["DestinyGender", int], str] = custom_field(
        metadata={"type": """dict[DestinyGender, str]"""}
    )
    titles_by_gender_hash: dict[int, str] = custom_field(metadata={"type": """dict[int, str]"""})
    manifest_gilding_tracking_record_hash: Optional["DestinyRecordDefinition"] = custom_field(default=None)


@custom_define()
class DestinyRecordCompletionBlock(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        partial_completion_objective_count_threshold: The number of objectives that must be completed before the objective is considered "complete"
        score_value: _No description given by bungie._
        should_fire_toast: _No description given by bungie._
        toast_style: _No description given by bungie._
    """

    partial_completion_objective_count_threshold: int = custom_field()
    score_value: int = custom_field()
    should_fire_toast: bool = custom_field()
    toast_style: Union["DestinyRecordToastStyle", int] = custom_field(
        converter=enum_converter("DestinyRecordToastStyle")
    )


@custom_define()
class SchemaRecordStateBlock(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        featured_priority: _No description given by bungie._
        obscured_description: A display description override to show when this record is 'obscured' instead of the default obscured display description.
        obscured_name: A display name override to show when this record is 'obscured' instead of the default obscured display name.
    """

    featured_priority: int = custom_field()
    obscured_description: str = custom_field()
    obscured_name: str = custom_field()


@custom_define()
class DestinyRecordExpirationBlock(BaseModel):
    """
    If this record has an expiration after which it cannot be earned, this is some information about that expiration.

    None
    Attributes:
        description: _No description given by bungie._
        has_expiration: _No description given by bungie._
        icon: _No description given by bungie._
    """

    description: str = custom_field()
    has_expiration: bool = custom_field()
    icon: str = custom_field()


@custom_define()
class DestinyRecordIntervalBlock(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        interval_objectives: _No description given by bungie._
        interval_rewards: _No description given by bungie._
        original_objective_array_insertion_index: _No description given by bungie._
    """

    interval_objectives: list["DestinyRecordIntervalObjective"] = custom_field(
        metadata={"type": """list[DestinyRecordIntervalObjective]"""}
    )
    interval_rewards: list["DestinyRecordIntervalRewards"] = custom_field(
        metadata={"type": """list[DestinyRecordIntervalRewards]"""}
    )
    original_objective_array_insertion_index: int = custom_field()


@custom_define()
class DestinyRecordIntervalObjective(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        interval_objective_hash: _No description given by bungie._
        interval_score_value: _No description given by bungie._
        manifest_interval_objective_hash: Manifest information for `interval_objective_hash`
    """

    interval_objective_hash: int = custom_field()
    interval_score_value: int = custom_field()
    manifest_interval_objective_hash: Optional["DestinyObjectiveDefinition"] = custom_field(default=None)


@custom_define()
class DestinyRecordIntervalRewards(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        interval_reward_items: _No description given by bungie._
    """

    interval_reward_items: list["DestinyItemQuantity"] = custom_field(
        metadata={"type": """list[DestinyItemQuantity]"""}
    )



from typing import TYPE_CHECKING, Union

from bungio.models.base import custom_define, custom_field
from bungio.models.mixins import DestinyCharacterMixin
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import BungieMembershipType

__all__ = ("DestinyCharacter",)


@custom_define()
class DestinyCharacter(DestinyCharacterMixin):
    """
    A representation of a Destiny 2 character

    Attributes:
        membership_id: The user's id
        membership_type: The user's type, aka platform
        character_id: The character's id
    """

    membership_id: int = custom_field()
    membership_type: Union["BungieMembershipType", int] = custom_field(converter=enum_converter("BungieMembershipType"))
    character_id: int = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import BaseModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import BungieMembershipType, DestinyItemComponent, DestinyMaterialRequirementSetDefinition


@custom_define()
class DestinyPlatformSilverComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        platform_silver: If a Profile is played on multiple platforms, this is the silver they have for each platform, keyed by Membership Type.
    """

    platform_silver: dict[Union["BungieMembershipType", int], "DestinyItemComponent"] = custom_field(
        metadata={"type": """dict[BungieMembershipType, DestinyItemComponent]"""}
    )


@custom_define()
class DestinyCurrenciesComponent(BaseModel):
    """
    This component provides a quick lookup of every item the requested character has and how much of that item they have. Requesting this component will allow you to circumvent manually putting together the list of which currencies you have for the purpose of testing currency requirements on an item being purchased, or operations that have costs. You *could* figure this out yourself by doing a GetCharacter or GetProfile request and forming your own lookup table, but that is inconvenient enough that this feels like a worthwhile (and optional) redundancy. Don't bother requesting it if you have already created your own lookup from prior GetCharacter/GetProfile calls.

    None
    Attributes:
        item_quantities: A dictionary - keyed by the item's hash identifier (DestinyInventoryItemDefinition), and whose value is the amount of that item you have across all available inventory buckets for purchasing. This allows you to see whether the requesting character can afford any given purchase/action without having to re-create this list itself.
        material_requirement_set_states: A map of material requirement hashes and their status information.
    """

    item_quantities: dict[int, int] = custom_field(metadata={"type": """dict[int, int]"""})
    material_requirement_set_states: dict[int, "DestinyMaterialRequirementSetState"] = custom_field(
        metadata={"type": """dict[int, DestinyMaterialRequirementSetState]"""}
    )


@custom_define()
class DestinyMaterialRequirementSetState(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        material_requirement_set_hash: The hash identifier of the material requirement set. Use it to look up the DestinyMaterialRequirementSetDefinition.
        material_requirement_states: The dynamic state values for individual material requirements.
        manifest_material_requirement_set_hash: Manifest information for `material_requirement_set_hash`
    """

    material_requirement_set_hash: int = custom_field()
    material_requirement_states: list["DestinyMaterialRequirementState"] = custom_field(
        metadata={"type": """list[DestinyMaterialRequirementState]"""}
    )
    manifest_material_requirement_set_hash: Optional["DestinyMaterialRequirementSetDefinition"] = custom_field(
        default=None
    )


@custom_define()
class DestinyMaterialRequirementState(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        count: The amount of the material required.
        item_hash: The hash identifier of the material required. Use it to look up the material's DestinyInventoryItemDefinition.
        stack_size: A value for the amount of a (possibly virtual) material on some scope. For example: Dawning cookie baking material requirements.
    """

    count: int = custom_field()
    item_hash: int = custom_field()
    stack_size: int = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import BaseModel, HashObject, ManifestModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import (
        ActivityGraphNodeHighlightType,
        DestinyActivityDefinition,
        DestinyDisplayPropertiesDefinition,
        DestinyGraphNodeState,
        DestinyObjectiveDefinition,
        DestinyPositionDefinition,
        DestinyUnlockExpressionDefinition,
    )


@custom_define()
class DestinyActivityGraphDefinition(ManifestModel, HashObject):
    """
    Represents a Map View in the director: be them overview views, destination views, or other. They have nodes which map to activities, and other various visual elements that we (or others) may or may not be able to use. Activity graphs, most importantly, have nodes which can have activities in various states of playability. Unfortunately, activity graphs are combined at runtime with Game UI-only assets such as fragments of map images, various in-game special effects, decals etc... that we don't get in these definitions. If we end up having time, we may end up trying to manually populate those here: but the last time we tried that, before the lead-up to D1, it proved to be unmaintainable as the game's content changed. So don't bet the farm on us providing that content in this definition.

    None
    Attributes:
        art_elements: Represents one-off/special UI elements that appear on the map.
        connections: Represents connections between graph nodes. However, it lacks context that we'd need to make good use of it.
        display_objectives: Objectives can display on maps, and this is supposedly metadata for that. I have not had the time to analyze the details of what is useful within however: we could be missing important data to make this work. Expect this property to be expanded on later if possible.
        display_progressions: Progressions can also display on maps, but similarly to displayObjectives we appear to lack some required information and context right now. We will have to look into it later and add more data if possible.
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        linked_graphs: Represents links between this Activity Graph and other ones.
        nodes: These represent the visual "nodes" on the map's view. These are the activities you can click on in the map.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
    """

    art_elements: list["DestinyActivityGraphArtElementDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityGraphArtElementDefinition]"""}
    )
    connections: list["DestinyActivityGraphConnectionDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityGraphConnectionDefinition]"""}
    )
    display_objectives: list["DestinyActivityGraphDisplayObjectiveDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityGraphDisplayObjectiveDefinition]"""}
    )
    display_progressions: list["DestinyActivityGraphDisplayProgressionDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityGraphDisplayProgressionDefinition]"""}
    )
    index: int = custom_field()
    linked_graphs: list["DestinyLinkedGraphDefinition"] = custom_field(
        metadata={"type": """list[DestinyLinkedGraphDefinition]"""}
    )
    nodes: list["DestinyActivityGraphNodeDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityGraphNodeDefinition]"""}
    )
    redacted: bool = custom_field()


@custom_define()
class DestinyActivityGraphNodeDefinition(BaseModel):
    """
    This is the position and other data related to nodes in the activity graph that you can click to launch activities. An Activity Graph node will only have one active Activity at a time, which will determine the activity to be launched (and, unless overrideDisplay information is provided, will also determine the tooltip and other UI related to the node)

    None
    Attributes:
        activities: The node may have various possible activities that could be active for it, however only one may be active at a time. See the DestinyActivityGraphNodeActivityDefinition for details.
        featuring_states: The node may have various visual accents placed on it, or styles applied. These are the list of possible styles that the Node can have. The game iterates through each, looking for the first one that passes a check of the required game/character/account state in order to show that style, and then renders the node in that style.
        node_id: An identifier for the Activity Graph Node, only guaranteed to be unique within its parent Activity Graph.
        override_display: The node *may* have display properties that override the active Activity's display properties.
        position: The position on the map for this node.
        states: Represents possible states that the graph node can be in. These are combined with some checking that happens in the game client and server to determine which state is actually active at any given time.
    """

    activities: list["DestinyActivityGraphNodeActivityDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityGraphNodeActivityDefinition]"""}
    )
    featuring_states: list["DestinyActivityGraphNodeFeaturingStateDefinition"] = custom_field(
        metadata={"type": """list[DestinyActivityGraphNodeFeaturingStateDefinition]"""}
    )
    node_id: int = custom_field()
    override_display: "DestinyDisplayPropertiesDefinition" = custom_field()
    position: "DestinyPositionDefinition" = custom_field()
    states: list["DestinyActivityGraphNodeStateEntry"] = custom_field(
        metadata={"type": """list[DestinyActivityGraphNodeStateEntry]"""}
    )


@custom_define()
class DestinyActivityGraphNodeFeaturingStateDefinition(BaseModel):
    """
    Nodes can have different visual states. This object represents a single visual state ("highlight type") that a node can be in, and the unlock expression condition to determine whether it should be set.

    None
    Attributes:
        highlight_type: The node can be highlighted in a variety of ways - the game iterates through these and finds the first FeaturingState that is valid at the present moment given the Game, Account, and Character state, and renders the node in that state. See the ActivityGraphNodeHighlightType enum for possible values.
    """

    highlight_type: Union["ActivityGraphNodeHighlightType", int] = custom_field(
        converter=enum_converter("ActivityGraphNodeHighlightType")
    )


@custom_define()
class DestinyActivityGraphNodeActivityDefinition(BaseModel):
    """
    The actual activity to be redirected to when you click on the node. Note that a node can have many Activities attached to it: but only one will be active at any given time. The list of Node Activities will be traversed, and the first one found to be active will be displayed. This way, a node can layer multiple variants of an activity on top of each other. For instance, one node can control the weekly Crucible Playlist. There are multiple possible playlists, but only one is active for the week.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: The activity that will be activated if the user clicks on this node. Controls all activity-related information displayed on the node if it is active (the text shown in the tooltip etc)
        node_activity_id: An identifier for this node activity. It is only guaranteed to be unique within the Activity Graph.
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_hash: int = custom_field()
    node_activity_id: int = custom_field()
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)


@custom_define()
class DestinyActivityGraphNodeStateEntry(BaseModel):
    """
    Represents a single state that a graph node might end up in. Depending on what's going on in the game, graph nodes could be shown in different ways or even excluded from view entirely.

    None
    Attributes:
        state: _No description given by bungie._
    """

    state: Union["DestinyGraphNodeState", int] = custom_field(converter=enum_converter("DestinyGraphNodeState"))


@custom_define()
class DestinyActivityGraphArtElementDefinition(BaseModel):
    """
    These Art Elements are meant to represent one-off visual effects overlaid on the map. Currently, we do not have a pipeline to import the assets for these overlays, so this info exists as a placeholder for when such a pipeline exists (if it ever will)

    None
    Attributes:
        position: The position on the map of the art element.
    """

    position: "DestinyPositionDefinition" = custom_field()


@custom_define()
class DestinyActivityGraphConnectionDefinition(BaseModel):
    """
    Nodes on a graph can be visually connected: this appears to be the information about which nodes to link. It appears to lack more detailed information, such as the path for that linking.

    None
    Attributes:
        dest_node_hash: _No description given by bungie._
        source_node_hash: _No description given by bungie._
    """

    dest_node_hash: int = custom_field()
    source_node_hash: int = custom_field()


@custom_define()
class DestinyActivityGraphDisplayObjectiveDefinition(BaseModel):
    """
    When a Graph needs to show active Objectives, this defines those objectives as well as an identifier.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        id: $NOTE $amola 2017-01-19 This field is apparently something that CUI uses to manually wire up objectives to display info. I am unsure how it works.
        objective_hash: The objective being shown on the map.
        manifest_objective_hash: Manifest information for `objective_hash`
    """

    id: int = custom_field()
    objective_hash: int = custom_field()
    manifest_objective_hash: Optional["DestinyObjectiveDefinition"] = custom_field(default=None)


@custom_define()
class DestinyActivityGraphDisplayProgressionDefinition(BaseModel):
    """
    When a Graph needs to show active Progressions, this defines those objectives as well as an identifier.

    None
    Attributes:
        id: _No description given by bungie._
        progression_hash: _No description given by bungie._
    """

    id: int = custom_field()
    progression_hash: int = custom_field()


@custom_define()
class DestinyLinkedGraphDefinition(BaseModel):
    """
    This describes links between the current graph and others, as well as when that link is relevant.

    None
    Attributes:
        description: _No description given by bungie._
        linked_graph_id: _No description given by bungie._
        linked_graphs: _No description given by bungie._
        name: _No description given by bungie._
        overview: _No description given by bungie._
        unlock_expression: _No description given by bungie._
    """

    description: str = custom_field()
    linked_graph_id: int = custom_field()
    linked_graphs: list["DestinyLinkedGraphEntryDefinition"] = custom_field(
        metadata={"type": """list[DestinyLinkedGraphEntryDefinition]"""}
    )
    name: str = custom_field()
    overview: str = custom_field()
    unlock_expression: "DestinyUnlockExpressionDefinition" = custom_field()


@custom_define()
class DestinyLinkedGraphEntryDefinition(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        activity_graph_hash: _No description given by bungie._
    """

    activity_graph_hash: int = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, Union

from bungio.models import SearchResultOfTrendingEntry, TrendingCategories, TrendingDetail, TrendingEntryType
from bungio.models.auth import AuthData
from bungio.models.base import ClientMixin, custom_define


@custom_define()
class TrendingRouteInterface(ClientMixin):
    async def get_trending_categories(self, auth: Optional[AuthData] = None) -> TrendingCategories:
        """
        Returns trending items for Bungie.net, collapsed into the first page of items per category. For pagination within a category, call GetTrendingCategory.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_trending_categories(auth=auth)
        return await TrendingCategories.from_dict(data=response, client=self._client, auth=auth)

    async def get_trending_category(
        self, category_id: str, page_number: int, auth: Optional[AuthData] = None
    ) -> SearchResultOfTrendingEntry:
        """
        Returns paginated lists of trending items for a category.

        Args:
            category_id: The ID of the category for whom you want additional results.
            page_number: The page # of results to return.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_trending_category(
            category_id=category_id, page_number=page_number, auth=auth
        )
        return await SearchResultOfTrendingEntry.from_dict(
            data=response, client=self._client, category_id=category_id, page_number=page_number, auth=auth
        )

    async def get_trending_entry_detail(
        self, identifier: str, trending_entry_type: Union[TrendingEntryType, int], auth: Optional[AuthData] = None
    ) -> TrendingDetail:
        """
        Returns the detailed results for a specific trending entry. Note that trending entries are uniquely identified by a combination of *both* the TrendingEntryType *and* the identifier: the identifier alone is not guaranteed to be globally unique.

        Args:
            identifier: The identifier for the entity to be returned.
            trending_entry_type: The type of entity to be returned.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_trending_entry_detail(
            identifier=identifier,
            trending_entry_type=getattr(trending_entry_type, "value", trending_entry_type),
            auth=auth,
        )
        return await TrendingDetail.from_dict(
            data=response,
            client=self._client,
            identifier=identifier,
            trending_entry_type=trending_entry_type,
            auth=auth,
        )



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, Union

from bungio.models import (
    FireteamDateRange,
    FireteamPlatform,
    FireteamPublicSearchOption,
    FireteamResponse,
    FireteamSlotSearch,
    SearchResultOfFireteamResponse,
    SearchResultOfFireteamSummary,
)
from bungio.models.auth import AuthData
from bungio.models.base import ClientMixin, custom_define


@custom_define()
class FireteamRouteInterface(ClientMixin):
    async def get_active_private_clan_fireteam_count(self, group_id: int, auth: AuthData) -> int:
        """
        Gets a count of all active non-public fireteams for the specified clan. Maximum value returned is 25.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            group_id: The group id of the clan.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_active_private_clan_fireteam_count(group_id=group_id, auth=auth)
        return response["Response"]

    async def get_available_clan_fireteams(
        self,
        activity_type: int,
        date_range: Union[FireteamDateRange, int],
        group_id: int,
        page: int,
        platform: Union[FireteamPlatform, int],
        public_only: Union[FireteamPublicSearchOption, int],
        slot_filter: Union[FireteamSlotSearch, int],
        auth: AuthData,
        exclude_immediate: Optional[bool] = None,
        lang_filter: Optional[str] = None,
    ) -> SearchResultOfFireteamSummary:
        """
        Gets a listing of all of this clan's fireteams that are have available slots. Caller is not checked for join criteria so caching is maximized.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            activity_type: The activity type to filter by.
            date_range: The date range to grab available fireteams.
            group_id: The group id of the clan.
            page: Zero based page
            platform: The platform filter.
            public_only: Determines public/private filtering.
            slot_filter: Filters based on available slots
            auth: Authentication information.
            exclude_immediate: If you wish the result to exclude immediate fireteams, set this to true. Immediate-only can be forced using the dateRange enum.
            lang_filter: An optional language filter.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_available_clan_fireteams(
            activity_type=activity_type,
            date_range=getattr(date_range, "value", date_range),
            group_id=group_id,
            page=page,
            platform=getattr(platform, "value", platform),
            public_only=getattr(public_only, "value", public_only),
            slot_filter=getattr(slot_filter, "value", slot_filter),
            auth=auth,
            exclude_immediate=exclude_immediate if exclude_immediate is not None else None,
            lang_filter=lang_filter if lang_filter is not None else None,
        )
        return await SearchResultOfFireteamSummary.from_dict(
            data=response,
            client=self._client,
            activity_type=activity_type,
            date_range=date_range,
            group_id=group_id,
            page=page,
            platform=platform,
            public_only=public_only,
            slot_filter=slot_filter,
            auth=auth,
            exclude_immediate=exclude_immediate,
            lang_filter=lang_filter,
        )

    async def search_public_available_clan_fireteams(
        self,
        activity_type: int,
        date_range: Union[FireteamDateRange, int],
        page: int,
        platform: Union[FireteamPlatform, int],
        slot_filter: Union[FireteamSlotSearch, int],
        auth: AuthData,
        exclude_immediate: Optional[bool] = None,
        lang_filter: Optional[str] = None,
    ) -> SearchResultOfFireteamSummary:
        """
        Gets a listing of all public fireteams starting now with open slots. Caller is not checked for join criteria so caching is maximized.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            activity_type: The activity type to filter by.
            date_range: The date range to grab available fireteams.
            page: Zero based page
            platform: The platform filter.
            slot_filter: Filters based on available slots
            auth: Authentication information.
            exclude_immediate: If you wish the result to exclude immediate fireteams, set this to true. Immediate-only can be forced using the dateRange enum.
            lang_filter: An optional language filter.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.search_public_available_clan_fireteams(
            activity_type=activity_type,
            date_range=getattr(date_range, "value", date_range),
            page=page,
            platform=getattr(platform, "value", platform),
            slot_filter=getattr(slot_filter, "value", slot_filter),
            auth=auth,
            exclude_immediate=exclude_immediate if exclude_immediate is not None else None,
            lang_filter=lang_filter if lang_filter is not None else None,
        )
        return await SearchResultOfFireteamSummary.from_dict(
            data=response,
            client=self._client,
            activity_type=activity_type,
            date_range=date_range,
            page=page,
            platform=platform,
            slot_filter=slot_filter,
            auth=auth,
            exclude_immediate=exclude_immediate,
            lang_filter=lang_filter,
        )

    async def get_my_clan_fireteams(
        self,
        group_id: int,
        include_closed: bool,
        page: int,
        platform: Union[FireteamPlatform, int],
        auth: AuthData,
        group_filter: Optional[bool] = None,
        lang_filter: Optional[str] = None,
    ) -> SearchResultOfFireteamResponse:
        """
        Gets a listing of all fireteams that caller is an applicant, a member, or an alternate of.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            group_id: The group id of the clan. (This parameter is ignored unless the optional query parameter groupFilter is true).
            include_closed: If true, return fireteams that have been closed.
            page: Deprecated parameter, ignored.
            platform: The platform filter.
            auth: Authentication information.
            group_filter: If true, filter by clan. Otherwise, ignore the clan and show all of the user's fireteams.
            lang_filter: An optional language filter.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_my_clan_fireteams(
            group_id=group_id,
            include_closed=include_closed,
            page=page,
            platform=getattr(platform, "value", platform),
            auth=auth,
            group_filter=group_filter if group_filter is not None else None,
            lang_filter=lang_filter if lang_filter is not None else None,
        )
        return await SearchResultOfFireteamResponse.from_dict(
            data=response,
            client=self._client,
            group_id=group_id,
            include_closed=include_closed,
            page=page,
            platform=platform,
            auth=auth,
            group_filter=group_filter,
            lang_filter=lang_filter,
        )

    async def get_clan_fireteam(self, fireteam_id: int, group_id: int, auth: AuthData) -> FireteamResponse:
        """
        Gets a specific fireteam.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            fireteam_id: The unique id of the fireteam.
            group_id: The group id of the clan.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_clan_fireteam(fireteam_id=fireteam_id, group_id=group_id, auth=auth)
        return await FireteamResponse.from_dict(
            data=response, client=self._client, fireteam_id=fireteam_id, group_id=group_id, auth=auth
        )



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import TYPE_CHECKING, Union

from bungio.models.base import BaseModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import PlatformErrorCodes


@custom_define()
class EntityActionResult(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        entity_id: _No description given by bungie._
        result: _No description given by bungie._
    """

    entity_id: int = custom_field(metadata={"int64": True})
    result: Union["PlatformErrorCodes", int] = custom_field(converter=enum_converter("PlatformErrorCodes"))



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import TYPE_CHECKING, Optional

from bungio.models.base import BaseModel, HashObject, ManifestModel, custom_define, custom_field

if TYPE_CHECKING:
    from bungio.models import (
        DestinyColor,
        DestinyDisplayPropertiesDefinition,
        DestinyInventoryItemDefinition,
        DestinyPresentationNodeDefinition,
        DestinyProgressionDefinition,
        DestinyVendorDefinition,
    )


@custom_define()
class DestinySeasonDefinition(ManifestModel, HashObject):
    """
    Defines a canonical "Season" of Destiny: a range of a few months where the game highlights certain challenges, provides new loot, has new Clan-related rewards and celebrates various seasonal events.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        acts: A list of Acts for the Episode
        artifact_item_hash: _No description given by bungie._
        background_image_path: _No description given by bungie._
        display_properties: _No description given by bungie._
        end_date: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        preview: Optional - Defines the promotional text, images, and links to preview this season.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        seal_presentation_node_hash: _No description given by bungie._
        season_number: _No description given by bungie._
        season_pass_hash: _No description given by bungie._
        season_pass_list: _No description given by bungie._
        season_pass_progression_hash: _No description given by bungie._
        seasonal_challenges_presentation_node_hash: _No description given by bungie._
        start_date: _No description given by bungie._
        manifest_artifact_item_hash: Manifest information for `artifact_item_hash`
        manifest_seal_presentation_node_hash: Manifest information for `seal_presentation_node_hash`
        manifest_season_pass_hash: Manifest information for `season_pass_hash`
        manifest_season_pass_progression_hash: Manifest information for `season_pass_progression_hash`
        manifest_seasonal_challenges_presentation_node_hash: Manifest information for `seasonal_challenges_presentation_node_hash`
    """

    acts: list["DestinySeasonActDefinition"] = custom_field(metadata={"type": """list[DestinySeasonActDefinition]"""})
    artifact_item_hash: int = custom_field()
    background_image_path: str = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    end_date: datetime = custom_field()
    index: int = custom_field()
    preview: "DestinySeasonPreviewDefinition" = custom_field()
    redacted: bool = custom_field()
    seal_presentation_node_hash: int = custom_field()
    season_number: int = custom_field()
    season_pass_hash: int = custom_field()
    season_pass_list: list["DestinySeasonPassReference"] = custom_field(
        metadata={"type": """list[DestinySeasonPassReference]"""}
    )
    season_pass_progression_hash: int = custom_field()
    seasonal_challenges_presentation_node_hash: int = custom_field()
    start_date: datetime = custom_field()
    manifest_artifact_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_seal_presentation_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_season_pass_hash: Optional["DestinySeasonPassDefinition"] = custom_field(default=None)
    manifest_season_pass_progression_hash: Optional["DestinyProgressionDefinition"] = custom_field(default=None)
    manifest_seasonal_challenges_presentation_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(
        default=None
    )


@custom_define()
class DestinySeasonPassReference(BaseModel):
    """
    Defines the hash, unlock flag and start time of season passes

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        season_pass_end_date: The Season Pass End Date
        season_pass_hash: The Season Pass Hash
        season_pass_start_date: The Season Pass Start Date
        manifest_season_pass_hash: Manifest information for `season_pass_hash`
    """

    season_pass_end_date: datetime = custom_field()
    season_pass_hash: int = custom_field()
    season_pass_start_date: datetime = custom_field()
    manifest_season_pass_hash: Optional["DestinySeasonPassDefinition"] = custom_field(default=None)


@custom_define()
class DestinySeasonPassDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        color: _No description given by bungie._
        display_properties: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        images: _No description given by bungie._
        index: The index of the entity as it was found in the investment tables.
        link_redirect_path: _No description given by bungie._
        prestige_progression_hash: I know what you're thinking, but I promise we're not going to duplicate and drown you. Instead, we're giving you sweet, sweet power bonuses.  Prestige progression is further progression that you can make on the Season pass after you gain max ranks, that will ultimately increase your power/light level over the theoretical limit.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        reward_progression_hash: This is the progression definition related to the progression for the initial levels 1-100 that provide item rewards for the Season pass. Further experience after you reach the limit is provided in the "Prestige" progression referred to by prestigeProgressionHash.
        manifest_prestige_progression_hash: Manifest information for `prestige_progression_hash`
        manifest_reward_progression_hash: Manifest information for `reward_progression_hash`
    """

    color: "DestinyColor" = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    images: "DestinySeasonPassImages" = custom_field()
    index: int = custom_field()
    link_redirect_path: str = custom_field()
    prestige_progression_hash: int = custom_field()
    redacted: bool = custom_field()
    reward_progression_hash: int = custom_field()
    manifest_prestige_progression_hash: Optional["DestinyProgressionDefinition"] = custom_field(default=None)
    manifest_reward_progression_hash: Optional["DestinyProgressionDefinition"] = custom_field(default=None)


@custom_define()
class DestinySeasonPassImages(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        icon_image_path: _No description given by bungie._
        theme_background_image_path: _No description given by bungie._
    """

    icon_image_path: str = custom_field()
    theme_background_image_path: str = custom_field()


@custom_define()
class DestinySeasonActDefinition(BaseModel):
    """
    Defines the name, start time and ranks included in an Act of an Episode.

    None
    Attributes:
        display_name: The name of the Act.
        rank_count: The number of ranks included in the Act.
        start_time: The start time of the Act.
    """

    display_name: str = custom_field()
    rank_count: int = custom_field()
    start_time: datetime = custom_field()


@custom_define()
class DestinySeasonPreviewDefinition(BaseModel):
    """
    Defines the promotional text, images, and links to preview this season.

    None
    Attributes:
        description: A localized description of the season.
        images: A list of images to preview the seasonal content. Should have at least three to show.
        link_path: A relative path to learn more about the season. Web browsers should be automatically redirected to the user's Bungie.net locale. For example: "/SeasonOfTheChosen" will redirect to "/7/en/Seasons/SeasonOfTheChosen" for English users.
        video_link: An optional link to a localized video, probably YouTube.
    """

    description: str = custom_field()
    images: list["DestinySeasonPreviewImageDefinition"] = custom_field(
        metadata={"type": """list[DestinySeasonPreviewImageDefinition]"""}
    )
    link_path: str = custom_field()
    video_link: str = custom_field()


@custom_define()
class DestinySeasonPreviewImageDefinition(BaseModel):
    """
    Defines the thumbnail icon, high-res image, and video link for promotional images

    None
    Attributes:
        high_res_image: An optional path to a high-resolution image, probably 1920x1080.
        thumbnail_image: A thumbnail icon path to preview seasonal content, probably 480x270.
    """

    high_res_image: str = custom_field()
    thumbnail_image: str = custom_field()


@custom_define()
class DestinyEventCardDefinition(ManifestModel, HashObject):
    """
    Defines the properties of an 'Event Card' in Destiny 2, to coincide with a seasonal event for additional challenges, premium rewards, a new seal, and a special title. For example: Solstice of Heroes 2022.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        color: _No description given by bungie._
        display_properties: _No description given by bungie._
        end_time: _No description given by bungie._
        event_card_currency_list: _No description given by bungie._
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        images: _No description given by bungie._
        index: The index of the entity as it was found in the investment tables.
        link_redirect_path: _No description given by bungie._
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        reward_progression_hash: _No description given by bungie._
        seal_presentation_node_hash: _No description given by bungie._
        ticket_currency_item_hash: _No description given by bungie._
        ticket_vendor_category_hash: _No description given by bungie._
        ticket_vendor_hash: _No description given by bungie._
        triumphs_presentation_node_hash: _No description given by bungie._
        weekly_challenges_presentation_node_hash: _No description given by bungie._
        manifest_reward_progression_hash: Manifest information for `reward_progression_hash`
        manifest_seal_presentation_node_hash: Manifest information for `seal_presentation_node_hash`
        manifest_ticket_currency_item_hash: Manifest information for `ticket_currency_item_hash`
        manifest_ticket_vendor_hash: Manifest information for `ticket_vendor_hash`
        manifest_triumphs_presentation_node_hash: Manifest information for `triumphs_presentation_node_hash`
        manifest_weekly_challenges_presentation_node_hash: Manifest information for `weekly_challenges_presentation_node_hash`
    """

    color: "DestinyColor" = custom_field()
    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    end_time: int = custom_field(metadata={"int64": True})
    event_card_currency_list: list[int] = custom_field(metadata={"type": """list[int]"""})
    images: "DestinyEventCardImages" = custom_field()
    index: int = custom_field()
    link_redirect_path: str = custom_field()
    redacted: bool = custom_field()
    reward_progression_hash: int = custom_field()
    seal_presentation_node_hash: int = custom_field()
    ticket_currency_item_hash: int = custom_field()
    ticket_vendor_category_hash: int = custom_field()
    ticket_vendor_hash: int = custom_field()
    triumphs_presentation_node_hash: int = custom_field()
    weekly_challenges_presentation_node_hash: int = custom_field()
    manifest_reward_progression_hash: Optional["DestinyProgressionDefinition"] = custom_field(default=None)
    manifest_seal_presentation_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_ticket_currency_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_ticket_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)
    manifest_triumphs_presentation_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)
    manifest_weekly_challenges_presentation_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(
        default=None
    )


@custom_define()
class DestinyEventCardImages(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        card_complete_image_path: _No description given by bungie._
        card_complete_wrap_image_path: _No description given by bungie._
        card_incomplete_image_path: _No description given by bungie._
        progress_icon_image_path: _No description given by bungie._
        theme_background_image_path: _No description given by bungie._
        unowned_card_sleeve_image_path: _No description given by bungie._
        unowned_card_sleeve_wrap_image_path: _No description given by bungie._
    """

    card_complete_image_path: str = custom_field()
    card_complete_wrap_image_path: str = custom_field()
    card_incomplete_image_path: str = custom_field()
    progress_icon_image_path: str = custom_field()
    theme_background_image_path: str = custom_field()
    unowned_card_sleeve_image_path: str = custom_field()
    unowned_card_sleeve_wrap_image_path: str = custom_field()



from typing import Callable, Coroutine, Optional

from bungio.http.route import Route
from bungio.models.auth import AuthData


class TokensRouteHttpRequests:
    request: Callable[..., Coroutine]

    async def force_drops_repair(self, auth: AuthData, *args, **kwargs) -> dict:
        """
        Twitch Drops self-repair function - scans twitch for drops not marked as fulfilled and resyncs them.

        Warning: Requires Authentication.
            Required oauth2 scopes: PartnerOfferGrant

        Args:
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/Tokens/Partner/ForceDropsRepair/", method="POST", auth=auth))

    async def claim_partner_offer(
        self, partner_offer_id: str, bungie_net_membership_id: int, transaction_id: str, auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Claim a partner offer as the authenticated user.

        Warning: Requires Authentication.
            Required oauth2 scopes: PartnerOfferGrant

        Args:
            partner_offer_id: _No description given by bungie._
            bungie_net_membership_id: _No description given by bungie._
            transaction_id: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "PartnerOfferId": partner_offer_id,
            "BungieNetMembershipId": bungie_net_membership_id,
            "TransactionId": transaction_id,
        }

        return await self.request(Route(path="/Tokens/Partner/ClaimOffer/", method="POST", data=data, auth=auth))

    async def apply_missing_partner_offers_without_claim(
        self, partner_application_id: int, target_bnet_membership_id: int, auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Apply a partner offer to the targeted user. This endpoint does not claim a new offer, but any already claimed offers will be applied to the game if not already.

        Warning: Requires Authentication.
            Required oauth2 scopes: PartnerOfferGrant

        Args:
            partner_application_id: The partner application identifier.
            target_bnet_membership_id: The bungie.net user to apply missing offers to. If not self, elevated permissions are required.
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Tokens/Partner/ApplyMissingOffers/{partner_application_id}/{target_bnet_membership_id}/",
                method="POST",
                auth=auth,
            )
        )

    async def get_partner_offer_sku_history(
        self, partner_application_id: int, target_bnet_membership_id: int, auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Returns the partner sku and offer history of the targeted user. Elevated permissions are required to see users that are not yourself.

        Warning: Requires Authentication.
            Required oauth2 scopes: PartnerOfferGrant

        Args:
            partner_application_id: The partner application identifier.
            target_bnet_membership_id: The bungie.net user to apply missing offers to. If not self, elevated permissions are required.
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Tokens/Partner/History/{partner_application_id}/{target_bnet_membership_id}/",
                method="GET",
                auth=auth,
            )
        )

    async def get_partner_reward_history(
        self, partner_application_id: int, target_bnet_membership_id: int, auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Returns the partner rewards history of the targeted user, both partner offers and Twitch drops.

        Warning: Requires Authentication.
            Required oauth2 scopes: PartnerOfferGrant

        Args:
            partner_application_id: The partner application identifier.
            target_bnet_membership_id: The bungie.net user to return reward history for.
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Tokens/Partner/History/{target_bnet_membership_id}/Application/{partner_application_id}/",
                method="GET",
                auth=auth,
            )
        )

    async def get_bungie_rewards_for_user(self, membership_id: int, auth: AuthData, *args, **kwargs) -> dict:
        """
        Returns the bungie rewards for the targeted user.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadAndApplyTokens

        Args:
            membership_id: bungie.net user membershipId for requested user rewards. If not self, elevated permissions are required.
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/Tokens/Rewards/GetRewardsForUser/{membership_id}/", method="GET", auth=auth)
        )

    async def get_bungie_rewards_for_platform_user(
        self, membership_id: int, membership_type: int, auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Returns the bungie rewards for the targeted user when a platform membership Id and Type are used.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadAndApplyTokens

        Args:
            membership_id: users platform membershipId for requested user rewards. If not self, elevated permissions are required.
            membership_type: The target Destiny 2 membership type.
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Tokens/Rewards/GetRewardsForPlatformUser/{membership_id}/{membership_type}/",
                method="GET",
                auth=auth,
            )
        )

    async def get_bungie_rewards_list(self, auth: Optional[AuthData] = None, *args, **kwargs) -> dict:
        """
        Returns a list of the current bungie rewards

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/Tokens/Rewards/BungieRewards/", method="GET", auth=auth))
